require 'net/http'
require 'json'
require 'base64'


class ColaClient
  MEDIA_TYPES = [:form, :json]
  METHODS = [:get, :post, :patch, :put, :delete]

  CONTENT_TYPE_HEADER = 'Content-Type'


  def self.go(method, url, query: nil, body: nil, headers: {}, options: {}) 
    uri = URI(url)
    uri.query = URI.encode_www_form(query) if query

    http = Net::HTTP.new(uri.host, uri.port)
    http.use_ssl = true if uri.scheme.match(/^https/)

    payload = nil
  
    if body
      case options[:content_type]
      when :form
        payload = URI.encode_www_form body
        headers[CONTENT_TYPE_HEADER] = 'application/x-www-form-urlencoded'
      when :json
        payload = body.to_json
        headers[CONTENT_TYPE_HEADER] = 'application/json'
      else
        headers[CONTENT_TYPE_HEADER] = 'text/plain'
        payload = body.to_s
      end
    end

    if options[:basic_auth]
      headers.merge! basic_auth options[:basic_auth]
    end

    response = http.send_request(method.to_s.upcase, uri.request_uri, payload, headers)

    if options[:accept] == :json
      response.body = JSON.parse(response.body, symbolize_names: true) unless response.body.empty?
    end

    response
  end

  def self.method_missing method, *args
    if METHODS.include? method.to_sym
      args.unshift method.to_sym
      self.send :go, *args
    end    
  end

  private

  def self.basic_auth(user_pass)
    credentials = "#{user_pass[:user]}:#{user_pass[:pass]}"
    encoded = Base64.strict_encode64(credentials)

    {
      authorization: "Basic #{encoded}"
    }
  end
end



# params = {
#   headers: {:haha => 'haha2'},
#   query: {b: 'c'},
#   body: {a: :b},
#   options: {
#     content_type: :json, 
#     accept: :json,
#     basic_auth: {
#       user: 'bla',
#       pass: 'bla2'
#     }
#   }
# }
# p ColaClient.get('http://localhost:4567/some/path', params)